/*---------------------------------------------------------------------------
Program :		AdvanceBar.cpp	Version 1.1
					TAdvanceBar Component for BCB1
Author :			Michel Leunen
Email :			michel@leunen.com
Date :			31/07/1998
Description :	Component for Borland C++ Builder V1 that draws an advance bar
					similar to the windows progress bar.
					It has 3 properties and 2 methods.
					Properties :	Position	gives or sets the position of the
										advance bar.
										Max		Position/Max gives the percentage of
										the drawing of the bar.
										Color		Color of the bar.
					Constructor :	TAdvanceBar	Assign default values to the
										properties.
					Destructor : 	~TAdvanceBar
					Methods :		Inc(int Value)	Increments the position
										Dec(int Value)	Decrements the position
History :	Version 1.1	Remove an annoying flickering effect when Position
				changes rapidly.
*/
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "AdvanceBar.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
static inline void ValidCtrCheck(TAdvanceBar *)
{
	new TAdvanceBar(NULL);
}
//---------------------------------------------------------------------------
__fastcall TAdvanceBar::TAdvanceBar(TComponent* Owner)
	: TGraphicControl(Owner)
{
    if(!ComponentState.Contains(csDesigning))	//if Designing mode
    {
    }
    FBitmap=NULL;
    FBitmap=new Graphics::TBitmap();
    SetBounds(Left,Top,80,15);              		//Set default properties Value
    FPosition=0;
    FBarColor=clDkGray;
    FMax=100;
    Textured=false;
    ControlStyle << csOpaque;							//avoid flickering
}
//---------------------------------------------------------------------------
__fastcall TAdvanceBar::~TAdvanceBar()
{
     delete FBitmap;
   FBitmap=NULL;
}
//---------------------------------------------------------------------------
namespace Advancebar
{
	void __fastcall PACKAGE Register()
	{
		TComponentClass classes[1] = {__classid(TAdvanceBar)};
		RegisterComponents("Leunen", classes, 0);
	}
}
//---------------------------------------------------------------------------
void __fastcall TAdvanceBar::SetBarColor(const TColor ABarColor)
{
	FBarColor=ABarColor;
	Repaint();
}
//---------------------------------------------------------------------------
void __fastcall TAdvanceBar::SetPosition (int APosition)
{
	FPosition=APosition;
   if(FPosition>FMax)FPosition=FMax;				//you can't exceed bounds
   if(FPosition<0)FPosition=0;
	Repaint();
}
//---------------------------------------------------------------------------
void __fastcall TAdvanceBar::SetMax (int AMax)
{
	FMax=AMax;
	Repaint();
}
//---------------------------------------------------------------------------
void __fastcall TAdvanceBar::Paint(void)
{
   //The bar is drawn first in a bitmap canvas and then copied into the
   //component canvas. This avoid flickering
	Graphics::TBitmap *PaintBitmap=new Graphics::TBitmap();
   //Bitmap canvas width and height=component width and height
   PaintBitmap->Width=Width;
   PaintBitmap->Height=Height;
   //Set background of bitmap
   PaintBitmap->Canvas->Brush->Color = TColor(clLtGray);
  	PaintBitmap->Canvas->Brush->Style = bsSolid;
  	PaintBitmap->Canvas->FillRect(Rect(0, 0, Width, Height));
	// Draw rect around the Advance Bar
   PaintBitmap->Canvas->Pen->Style=psSolid;
   PaintBitmap->Canvas->Pen->Color=clBlack;
   PaintBitmap->Canvas->LineTo(Width-1,0);
	PaintBitmap->Canvas->Pen->Color=clWhite;
   PaintBitmap->Canvas->LineTo(Width-1,Height-1);
	PaintBitmap->Canvas->LineTo(0,Height-1);
   PaintBitmap->Canvas->Pen->Color=clBlack;
   PaintBitmap->Canvas->LineTo(0,0);
   // Draw the bar itself
   if(FTextured)
   {
		TRect	BarRect;
      BarRect.Left=2;
      BarRect.Top=2;
      BarRect.Right=((FPosition*(Width-4))/FMax)+2;
      BarRect.Bottom=Height-2;
      TileBitmap(BarRect,PaintBitmap);
   }
   else
   {
		PaintBitmap->Canvas->Brush->Style=bsSolid;
   	PaintBitmap->Canvas->Brush->Color=FBarColor;
   	PaintBitmap->Canvas->Pen->Color=PaintBitmap->Canvas->Brush->Color;
   	PaintBitmap->Canvas->Rectangle(2,2,((FPosition*(Width-4))/FMax)+2,Height-2);
   }
   //copy the bitmap into the component canvas
   Canvas->CopyMode=cmSrcCopy;
   Canvas->CopyRect(Rect(0,0,Width,Height),
   					  PaintBitmap->Canvas,Rect(0,0,Width,Height));
   //bitmap is no more needed
   delete PaintBitmap;
}
//---------------------------------------------------------------------------
void __fastcall TAdvanceBar::Inc(int Value)
{
	FPosition += Value;
   if(FPosition>FMax)FPosition=FMax;
   Repaint();
}
//---------------------------------------------------------------------------
void __fastcall TAdvanceBar::Dec(int Value)
{
	FPosition -=Value;
   if(FPosition<0)FPosition=0;
   Repaint();
}
//---------------------------------------------------------------------------
void __fastcall TAdvanceBar::SetBitmap(Graphics::TBitmap *ABitmap)
{
	FBitmap->Assign(ABitmap);
   Repaint();
}
//---------------------------------------------------------------------------
void __fastcall TAdvanceBar::TileBitmap(TRect rect,Graphics::TBitmap *PBitmap)
{
   int i,j,bmpWidth,bmpHeight;
   
	for(i=rect.Top;i<rect.Bottom;i+=FBitmap->Height)
   {
   	for(j=rect.Left;j<rect.Right;j+=FBitmap->Width)
      {
      	if(j<rect.Right-FBitmap->Width)bmpWidth=FBitmap->Width;
         else bmpWidth=rect.Right-j;
         if(i<rect.Bottom-FBitmap->Height)bmpHeight=FBitmap->Height;
         else bmpHeight=rect.Bottom-i;
         ::BitBlt(PBitmap->Canvas->Handle,   //Handle of dest. device context
         j,                        		//x coord. of dest. upper-left corner
         i,                        		//y coord. of dest. upper-left corner
         bmpWidth,                 		//width of dest. rectangle
         bmpHeight,                		//height of dest. rectangle
         FBitmap->Canvas->Handle,			//Handle of source device context
         0,                        		//x coord of source upper-left corner
         0,                        		//y coord of source upper-left corner
         SRCCOPY);                 		//Specifies how to draw
      }
   }
}
//---------------------------------------------------------------------------
